require 'mini_magick'
require 'presto/transformation'
require 'presto/dimensions'
require 'presto/exceptions'

module Presto
	module MiniMagick
		class Processor
			def self.process image, params={}
				new(params).process(image)
			end
			
			attr_reader :transformation

			def initialize params={}
				@transformation = ::Presto::Transformation.new(params)
			end

			def process image
				return image if trans.empty?
				trans.validate!
				process_fit(image) if trans[:w] || trans[:h]
				process_format(image) if trans[:fm]
				process_quality(image) if trans[:q]
				image
			rescue ::Veto::InvalidEntity => e
				raise(::Presto::InvalidTransformation, e.message)
			end

			private

			def process_fit image
				send("process_#{trans[:fit]}_fit", image)
			end

			def process_crop_fit image
				d = replaced_dimensions(image)
				image.combine_options do |cmd|
					cmd.resize "#{d.width}x#{d.height}^"
					cmd.background trans[:bg]
					cmd.gravity 'Center'
					cmd.extent "#{d.width}x#{d.height}"
				end
			end

			def process_clip_fit image
				d = resized_dimensions(image)
				image.resize "#{d.width}x#{d.height}"
			end

			def process_scale_fit image
				d = replaced_dimensions(image)
				image.resize "#{d.width}x#{d.height}!"
			end

			def process_max_fit image
				d = replaced_dimensions(image)
				image.resize "#{d.width}x#{d.height}>"
			end

			def process_fill_fit image
				d = replaced_dimensions(image)
				image.combine_options do |cmd|
					cmd.resize "#{d.width}x#{d.height}"
					cmd.background trans[:bg]
					cmd.gravity 'Center'
					cmd.extent "#{d.width}x#{d.height}"
				end
			end

			def process_format image
				image.format trans[:fm]
			end

			def process_quality image
				image.quality trans[:q]
			end

			def resized_dimensions image
				::Presto::Dimensions.resize(image[:width], image[:height], trans[:w], trans[:h])
			end

			def replaced_dimensions image
				w = trans[:w] || image[:width]
				h = trans[:h] || image[:height]
				::Presto::Dimensions.new w, h
			end

			def trans
				transformation
			end
		end

		module Extension
			def presto transformation={}
				::Presto::MiniMagick::Processor.process(self, transformation)
			end
		end
	end
end

::MiniMagick::Image.send(:include, ::Presto::MiniMagick::Extension)