require 'presto'
require 'veto'

module Presto
	class Transformation
		class Validator
			HEXDECIMAL_ALPHA_REGEX = /^#[a-fA-F0-9]{8}$/
			HEXDECIMAL_LONG_REGEX = /^#[a-fA-F0-9]{6}$/
			HEXDECIMAL_SHORT_REGEX = /^#[a-fA-F0-9]{3}$/
			FIT_TYPES = %w(clip crop scale fill max).freeze
			QUALITY_MAX = 100
			QUALITY_MIN = 1

			include Veto.validator

			def self.config
				Presto.configuration
			end

			validates :w, :greater_than_or_equal_to => config.min_width, :less_than_or_equal_to => config.max_width, :unless => "w.nil?"
			validates :h, :greater_than_or_equal_to => config.min_height, :less_than_or_equal_to => config.max_height, :unless => "h.nil?"
			validates :fit, :presence => true, :inclusion => FIT_TYPES
			validate :validate_bg_format, :unless => "bg.nil?"
			validates :fm, :inclusion => config.extension_white_list, :unless => "fm.nil?"
			validates :q, :greater_than_or_equal_to => QUALITY_MIN, :less_than_or_equal_to => QUALITY_MAX, :unless => "q.nil?"

			private

			def validate_bg_format
				unless entity.bg =~ HEXDECIMAL_LONG_REGEX || entity.bg =~ HEXDECIMAL_SHORT_REGEX || entity.bg =~ HEXDECIMAL_ALPHA_REGEX 
					errors.add(:bg, "is not valid")
				end
			end
		end
	end
end